/*
 * (c) 1997-2011 Netflix, Inc.  All content herein is protected by
 * U.S. copyright and other applicable intellectual property laws and
 * may not be copied without the express permission of Netflix, Inc.,
 * which reserves all rights.  Reuse of any of this content for any
 * purpose without the permission of Netflix, Inc. is strictly
 * prohibited.
 */

var nrdp = window.nrdp;

(function() {

if (typeof window.nrdp !== "undefined") {
    return;
}

nrdp = {
    _URL: "http://localcontrol.netflix.com/",
    _filter: "ui",
    _iframeSrc: "html/iframe.html",
    _isReady: false,
    _isError: false,
    _syncData: {
        nrdp: {},
        device: {},
        registration: {},
        storage: {},
        log: {},
        media: {}
    },
    _pendingSetProp: {},
    _currentSetProp: {},
    _eventListeners: [],

    get isReady() { return nrdp._isReady; },
    get debug() { return nrdp._syncData.nrdp.debug; },
    get bootURL() { return nrdp._syncData.nrdp.bootUrl; },

    get location() { return window.location; },
    set location(e) { nrdp._invoke("system", "setLocation", {location: e}); },
    //set location(e) { window.location = e; },

    get READY() { return "READY"; },

    get COMPLETE() { return "COMPLETE"; },
    get NETWORK_ERROR() { return "NETWORK_ERROR"; },
    get ACTION_ID() { return "ACTION_ID"; },
    get ERROR() { return "ERROR"; },

    _gotMessage: function(e) {
        if (e.origin + "/" !== nrdp._URL) {
            return;
        }

        nrdp._gotJSON(e.data.data);
    },

    _gotJSON: function(jsonStr) {
        var event;
        try {
            event = JSON.parse(jsonStr);
        } catch (err) {
            console.log("unhandled exception in eventsource " + jsonStr);
            console.log(err.toString());
            return;
        }
        if (event.type == "Event") {
            if (!nrdp._handleEvent(event))
                console.log("unhandled event " + jsonStr);
        } else if (event.type == "PropertyUpdate") {
            for (var property in event.properties) {
                nrdp._updateProperty(event.object, property, event.properties[property]);
            }
        } else if (event.type == "EventSourceError") {
            var evt = {
                type: "fatalerror"
            };
            nrdp._isError = true;
            console.log("EventSource went away, sending fatalerror");
            nrdp._callEventListeners("nrdp", evt);
        } else {
            console.log("unhandled eventsource type " + jsonStr);
        }
    },

    init: function(url) {
        if (nrdp._init) {
            return;
        }
        nrdp.log.trace("init called, window.location = " + window.location);
        nrdp._init = true;
        if (url) {
            nrdp._URL = url;
        } else {
            for (var i in document.scripts) {
                if (/^http.*js\/NetflixBridge.js$/.test(document.scripts[i].src)) {
                    var path = document.scripts[i].src.replace("js/NetflixBridge.js", "");
                    var xhr = new XMLHttpRequest();
                    xhr.open("GET", path + "html/error.html", false);
                    xhr.send();
                    if (xhr.readyState == 4 && xhr.status == 200) {
                        nrdp.log.trace("using " + path + " for nbpd URL");
                        nrdp._URL = path;
                    }
                }
            }
        }

        try {
            var evtsrc = new EventSource(nrdp._URL + "eventsource?objectFilter=" + nrdp._filter);
            evtsrc.onmessage = function(evt) { nrdp._gotJSON(evt.data); };
            evtsrc.onerror = function(evt) {
                evtsrc.close();
                nrdp._gotJSON('{"type": "EventSourceError"}');
            };
        } catch (x) {
            nrdp.log.trace("Direct EventSource connection failed, using iframe");
            window.addEventListener("message", nrdp._gotMessage, false);

            var iframe = document.createElement('iframe');
            iframe.style.setProperty("opacity", 0);
            iframe.width = iframe.height = 0;
            iframe.src = nrdp._URL + nrdp._iframeSrc;
            document.body.appendChild(iframe);
        }
    },

    exit: function() {
        nrdp._invoke(null, "quit");
    },

    getConfigList: function() {
        nrdp._invoke(null, "getConfigList", null,
                     function(resp) {
                         var evt = {
                             type: "config"
                         };
                         try {
                             evt.list = JSON.parse(resp).returnValue;
                         } catch (x) {
                             // ignore
                         }
                         if (evt.list && evt.list.length) {
                             nrdp._callEventListeners("nrdp", evt);
                         }
                     });
    },

    setConfigData: function(file, data) {
        var str = nrdp._urlEncode(data);
        if (str)
            nrdp._invoke(null, "setConfigData", {name: file, data: str});
    },

    addEventListener: function(evt, listener) { nrdp._addEventListener("nrdp", evt, listener); },
    removeEventListener: function(evt, listener) { nrdp._removeEventListener("nrdp", evt, listener); },

    _addEventListener: function(prefix, eventType, listener) {
        eventType = prefix + "." + eventType;
        nrdp.log.trace("add listener for " + eventType);

        if (listener) {
            if (!this._eventListeners[eventType]) {
                this._eventListeners[eventType] = [];
            }
            this._eventListeners[eventType].push(listener);
        }
    },

    _removeEventListener: function(prefix, eventType, listener) {
        eventType = prefix + "." + eventType;
        nrdp.log.trace("remove listener for " + eventType);

        if (listener) {
            var myListeners = this._eventListeners[eventType];
            if (myListeners) {
                var index = myListeners.indexOf(listener);
                if (index != undefined) {
                    if (myListeners.length == 1)
                        this._eventListeners[eventType] = undefined;
                    else
                        this._eventListeners[eventType].splice(index, 1);
                }
            }
        }
    },

    _hasEventListener: function(prefix, eventType) {
        eventType = prefix + "." + eventType;
        return (nrdp._eventListeners[eventType] && nrdp._eventListeners[eventType].length);
    },

    _callEventListeners: function(prefix, event) {
        if (!this.isReady) {
            return;
        }
        var eventType = prefix + "." + event.type;
        if (this._eventListeners[eventType]) {
            var myListeners = this._eventListeners[eventType].slice(0);
            for (var i = 0; i < myListeners.length; i++) {
                var listener = myListeners[i];
                if (listener) {
                    nrdp.log.trace("call listener for " + eventType);
                    listener(event);
                }
            }
        }
    },

    _POST: function(params, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open("POST", this._URL + "ipc", true);
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        xhr.onload = function() { onload(xhr.responseText); };
        xhr.onerror = onerror;
        xhr.send(params);
    },

    _urlEncode: function(obj) {
        var str = "";
        for (var p in obj) {
            if (str) str += "&";
            str += p + "=" + encodeURIComponent(obj[p]);
        }
        return str;
    },

    _setProperty: function(subobj, prop, val) {
        var obj = subobj ? "nrdp." + subobj : "nrdp";
        var full = obj + "." + prop;
        if (this._currentSetProp[full]) {
            nrdp.log.trace("setting " + full + " while outstanding");
            this._pendingSetProp[full] = {
                object: subobj,
                property: prop,
                value: val
            };
            return;
        }
        this._currentSetProp[full] = true;
        var params = "object=" + obj + "&property=" + prop + "&value=" + encodeURIComponent(val);

        this._POST(params,
                   function() {
                       nrdp._currentSetProp[full] = false;
                       if (nrdp._pendingSetProp[full]) {
                           var newset = nrdp._pendingSetProp[full];
                           nrdp._pendingSetProp[full] = undefined;
                           nrdp._setProperty(newset.object, newset.property, newset.value);
                       }
                   },
                   function() {
                       console.log("lost IPC connection for setProperty(" + obj + "." + prop + ")");
                   });
    },

    _invoke: function(obj, method, args, cb) {
        obj = obj ? "nrdp." + obj : "nrdp";
        var params = "object=" + obj + "&method=" + method;
        for (var a in args) {
            params += "&" + a + "=" + encodeURIComponent(args[a]);
        }

        this._POST(params,
                   function(text) {
                       //nrdp.log.trace(text);
                       if (cb) cb(text);
                   },
                   function() {
                       console.log("lost IPC connection for invoke(" + obj + "." + method + ")");
                   });
    },

    _handleEvent: function(event) {
        if (!event.object) {
            if (event.name == "ObjectSyncComplete") {
                if (!this._isReady) {
                    this._isReady = true;
                    var evt = {
                        type: "init",
                        status: this.READY
                    };
                    this._callEventListeners("nrdp", evt);
                }
                return true;
            }
        } else if (event.object == "nrdp") {
            if (event.name == "background") {
                return this._handleNccpEvent(event, "nrdp", "background");
            } else if (event.name == "commandReceived") {
                this._callEventListeners("nrdp", { type: "command", parameters: event.data });
                return true;
            }
        } else if (event.object == "nrdp.device") {
            return this.device._handleEvent(event);
        } else if (event.object == "nrdp.registration") {
            return this.registration._handleEvent(event);
        } else if (event.object == "nrdp.log") {
            return this.log._handleEvent(event);
        } else if (event.object == "nrdp.media") {
            return this.media._handleEvent(event);
        }
        return false;
    },

    _handleNccpEvent: function(event, object, type) {
        var evt = {
            type: type
        };
        if (event.data.origin == "complete") {
            return true;
        } else if (event.data.origin == "reportNetworkingError") {
            evt.result = this.NETWORK_ERROR;
        } else if (event.data.origin == "stopTransaction" || event.data.origin == "handleActionId") {
            evt.result = this.ACTION_ID;
            evt.actionID = event.data.actionId;
            evt.message = event.data.message;
            evt.bcp47 = event.data.bcp47;
            evt.reasonCode = event.data.reasonCode;
        } else {
            return false;
        }
        this._callEventListeners(object, evt);
        return true;
    },

    _updateProperty: function(object, property, value) {
        if (object == "nrdp.device") {
            this.device._updateProperty(property, value);
        } else if (object == "nrdp.log") {
            this.log._updateProperty(property, value);
        } else if (object == "nrdp.storage") {
            this.storage._updateProperty(property, value);
        } else {
            this._syncData[object.replace("nrdp.", "")][property] = value;
        }
    }
};

nrdp.device = {
    addEventListener: function(evt, listener) { nrdp._addEventListener("device", evt, listener); },
    removeEventListener: function(evt, listener) { nrdp._removeEventListener("device", evt, listener); },

    get softwareVersion() { return nrdp._syncData.device.softwareVersion; },
    get certificationVersion() { return nrdp._syncData.device.certificationVersion; },
    get ESNPrefix() { return nrdp._syncData.device.ESNPrefix; },
    get SDKVersion() { return nrdp._syncData.device.SDKVersion; },
    get ESN() { return nrdp._syncData.device.ESN; },
    get language() { return nrdp._syncData.device.language; },

    get parentalControlRating() { return nrdp._syncData.device.parentalControlRating; },
    get PC_NONE() { return 0; },
    get PC_PIN() { return 1; },
    get PC_NATIVE() { return 2; },
    get parentalControlOverrideType() { return nrdp._syncData.device.parentalControlOverrideType; },

    get capability() { return nrdp._syncData.device.capability; },
    get videoOutput() { return nrdp._syncData.device.videoOutput; },

    setUIVersion: function(version) { nrdp._setProperty("device", "UIVersion", version); },

    get UILanguages() {
        return nrdp._syncData.device.UILanguages;
    },
    set UILanguages(langs) {
        var val = "";
        for (var i = 0; i < langs.length; i++) {
            if (val) val += ",";
            val += encodeURIComponent(langs[i]);
        }
        nrdp._setProperty("device", "UILanguages", val);
    },

    isScreensaverOn: function() { return nrdp._syncData.device.screensaverOn; },

    checkParentalControlPINCode: function(pin, cb) {
        if (nrdp.device.parentalControlOverrideType != nrdp.device.PC_PIN)
            return false;
        nrdp.device._pendingPCCB = cb;
        nrdp._invoke("device", "checkParentalControlPIN", {pin: pin});
        return true;
    },
    checkParentalControlMovieRating: function(rating, cb) {
        if (nrdp.device.parentalControlOverrideType != nrdp.device.PC_NATIVE)
            return false;
        nrdp.device._pendingPCCB = cb;
        nrdp._invoke("device", "showParentalControlAuthDialog", {rating: rating});
        return true;
    },

    _handleEvent: function(event) {
        if (event.name == "notifyParentalControlAuthResult") {
            if (this._pendingPCCB) {
                this._pendingPCCB(event.data);
                this._pendingPCCB = undefined;
            }
        } else {
            return false;
        }
        return true;
    },

    _updateProperty: function(property, value) {
        var evt;
        if (property == "screensaverOn") {
            evt = {
                type: "screensaverchange"
            };
        } else if (property == "capability") {
            evt = {
                type: "capabilitychange",
                old: this.capability
            };
        } else if (property == "videoOutput") {
            evt = {
                type: "videooutputchange",
                old: this.videoOutput
            };
        } else if (property == "language") {
            evt = {
                type: "languagechange",
                old: this.language
            };
        } else if (property == "parentalControlRating") {
            evt = {
                type: "parentalcontrols",
                old: this.parentalControlRating
            };
        }

        nrdp._syncData.device[property] = value;
        if (evt) {
            nrdp._callEventListeners("device", evt);
        }
    }
};

nrdp.registration = {
    addEventListener: function(evt, listener) { nrdp._addEventListener("registration", evt, listener); },
    removeEventListener: function(evt, listener) { nrdp._removeEventListener("registration", evt, listener); },

    get deviceAccounts() { return nrdp._syncData.registration.deviceAccounts; },
    get currentDeviceAccount() { return nrdp._syncData.registration.currentDeviceAccount; },

    get registered() { return nrdp._syncData.registration.registered; },

    get activationTokens() {
        for (var i = 0; i < nrdp.registration.deviceAccounts.length; i++) {
            if (nrdp.registration.deviceAccounts[i].accountKey === nrdp.registration.currentDeviceAccount) {
                return nrdp.registration.deviceAccounts[i].tokens;
            }
        }
        return {};
    },
    set activationTokens(tokens) {
        nrdp.registration._validateDak(nrdp.registration.currentDeviceAccount);
        nrdp._invoke("registration", "setActivationTokens", tokens);
    },

    startRendezvous: function() {
        nrdp.registration._validateDak(nrdp.registration.currentDeviceAccount);
        nrdp._invoke("registration", "startRendezvous");
    },
    cancelRendezvous: function() { nrdp._invoke("registration", "cancelRendezvous"); },

    createDeviceAccount: function(cb) {
        nrdp._invoke("registration", "createDeviceAccount", null,
                     function(result) {
                         var obj;
                         try {
                             obj = JSON.parse(result).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to createDeviceAccount: " + result);
                             obj = {
                                 result: nrdp.ERROR,
                                 code: -1
                             };
                         }
                         if (obj.result == nrdp.COMPLETE) {
                             nrdp._syncData.registration.deviceAccounts = obj.deviceAccounts;
                         }
                         if (cb) cb(obj);
                     });
    },
    selectDeviceAccount: function(dak, cb) {
        nrdp.registration._validateDak(dak);
        nrdp._invoke("registration", "selectDeviceAccount", {key: dak},
                     function(result) {
                         var obj;
                         try {
                             obj = JSON.parse(result).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to selectDeviceAccount: " + result);
                         }
                         if (obj.result == nrdp.COMPLETE) {
                             nrdp._syncData.registration.deviceAccounts = obj.deviceAccounts;
                             nrdp._syncData.registration.currentDeviceAccount = obj.key;
                             nrdp._syncData.registration.registered = obj.registered;
                         }
                         if (cb) cb();
                         nrdp._invoke("registration", "ping");
                         // if the selected device account is registered, get the list of config service
                         // files to retrieve
                         if (nrdp.registration.registered)
                             nrdp.getConfigList();
                     });
    },
    unselectDeviceAccount: function(cb) {
        nrdp._invoke("registration", "unselectDeviceAccount", null,
                     function(result) {
                         nrdp._syncData.registration.registered = false;
                         nrdp._syncData.registration.currentDeviceAccount = null;
                         if (cb) cb();
                     });
    },

    _validateDak: function(dak) {
        for (var i = 0; this.deviceAccounts && i < this.deviceAccounts.length; i++) {
            if (this.deviceAccounts[i].accountKey == dak)
                return;
        }
        throw "cannot find device account key '" + dak + "'";
    },

    getDeviceTokens: function() {
        nrdp._invoke("registration", "getDeviceTokens", null,
                     function(resp) {
                         var obj;
                         try {
                             obj = JSON.parse(resp).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to getDeviceTokens: " + rv);
                         }
                         var evt = {
                             type: "bind",
                             result: nrdp.COMPLETE,
                             cookies: obj
                         };
                         nrdp._callEventListeners("registration", evt);
                     });
    },

    _activate: function(type, params) {
        this._validateDak(this.currentDeviceAccount);
        nrdp._invoke("registration", type + "Activate", params,
                     // the NCCP transaction completes before auth is updated, meaning
                     // we get the COMPLETE event from c++ before nrdp.registration.registered
                     // is updated. So when tokenActivate returns is when we send this event.
                     function(rv) {
                         var obj;
                         try {
                             obj = JSON.parse(rv).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to tokenActivate: " + rv);
                         }
                         if (!obj) {
                             nrdp.log.trace("invalid response to activate");
                             return;
                         }
                         nrdp._syncData.registration.registered = obj.registered;
                         nrdp._syncData.registration.deviceAccounts = obj.deviceAccounts;
                         var evt = {
                             type: "activate",
                             result: nrdp.COMPLETE,
                             cookies: obj.cookies
                         };
                         nrdp._callEventListeners("registration", evt);
                         // if the selected device account is registered, get the list of config service
                         // files to retrieve
                         if (nrdp.registration.registered)
                             nrdp.getConfigList();
                     });
    },
    tokenActivate: function(tokens) {
        nrdp.registration._activate("token", tokens);
    },
    emailActivate: function(email, passwd) {
        nrdp.registration._activate("email", {email: email, passwd: passwd});
    },

    massDeactivationCheck: function() {
        nrdp._invoke("registration", "massDeactivationCheck");
    },

    deactivate: function(dak, cb) {
        nrdp.storage.clear(dak);
        nrdp._invoke("registration", "deactivate", {key: dak},
                     function(result) {
                         var obj = {};
                         try {
                             obj = JSON.parse(result).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to deactivate: " + result);
                         }
                         if (obj.result == nrdp.COMPLETE) {
                             nrdp._syncData.registration.deviceAccounts = obj.deviceAccounts;
                             nrdp._syncData.registration.currentDeviceAccount = obj.key;
                             nrdp._syncData.registration.registered = obj.registered;
                         }
                         if (cb) cb();
                     });
    },
    deactivateAll: function(cb) {
        nrdp.storage._clearAll();
        nrdp._invoke("registration", "deactivateAll", null,
                     function(result) {
                         nrdp._syncData.registration.deviceAccounts = [];
                         nrdp._syncData.registration.currentDeviceAccount = null;
                         nrdp._syncData.registration.registered = false;
                         if (cb) cb();
                     });
    },

    _handleEvent: function(event) {
        if (event.name == "ping") {
            // ping is not exposed as a public API, it's done implicitly at device account selection
            // to handle the ebay case, failures show up as background events
            return nrdp._handleNccpEvent(event, "nrdp", "background");
        } else if (event.name == "rendezvous") {
            var evt = {
                type: "rendezvous",
                code: event.data.code,
                registered: event.data.origin == "success",
                cookies: event.data.cookies
            };
            if (event.data.origin == "setRegCode" || event.data.origin == "success") {
                evt.result = nrdp.COMPLETE;
            } else if (event.data.origin == "failure") {
                evt.result = nrdp.ACTION_ID;
                evt.actionID = event.data.actionId;
                evt.message = event.data.message;
                evt.bcp47 = event.data.bcp47;
                evt.reasonCode = event.data.reasonCode;
            } else if (event.data.origin == "networkError") {
                evt.result = nrdp.NETWORK_ERROR;
            } else {
                return false;
            }
            nrdp._callEventListeners("registration", evt);
            // if the selected device account is registered, get the list of config service
            // files to retrieve
            if (nrdp.registration.registered)
                nrdp.getConfigList();
        } else if (event.name == "bind") {
            return nrdp._handleNccpEvent(event, "registration", "bind");
        } else if (event.name == "tokenActivate" || event.name == "emailActivate") {
            return nrdp._handleNccpEvent(event, "registration", "activate");
        } else if (event.name == "deactivated") {
            var evt = {
                type: "deactivated",
                accountKey: event.data.key,
                current: (this.currentDeviceAccount === event.data.key)
            };
            nrdp._syncData.registration.deviceAccounts = event.data.deviceAccounts;
            nrdp._syncData.registration.currentDeviceAccount = event.data.currentDeviceAccount;
            nrdp._syncData.registration.registered = event.data.registered;
            nrdp._callEventListeners("registration", evt);
        } else {
            return false;
        }

        return true;
    }
};

nrdp.storage = {
    get NO_DEVICE_ACCOUNT() { return "NDAKADN"; },

    get size() { return JSON.stringify(nrdp._syncData.storage.data).length; },

    length: function(dak) {
        var data = nrdp.storage._getData(dak);
        var count = 0;
        for (var key in data) {
            if (data[key] != undefined)
                count++;
        }
        return count;
    },

    key: function(dak, n) {
        var data = nrdp.storage._getData(dak);
        for (var key in data) {
            if (data[key] == undefined)
                continue;
            if (n == 0)
                return key;
            n--;
        }
        return undefined;
    },

    getItem: function(dak, key) {
        var data = nrdp.storage._getData(dak);
        return data[key];
    },

    setItem: function(dak, key, value) {
        var data = nrdp.storage._getData(dak);
        data[key] = value;
        nrdp._setProperty("storage", "data", JSON.stringify(nrdp._syncData.storage.data));
    },

    removeItem: function(dak, key) {
        var data = nrdp.storage._getData(dak);
        delete data[key];
        nrdp._setProperty("storage", "data", JSON.stringify(nrdp._syncData.storage.data));
    },

    clear: function(dak) {
        var data = nrdp.storage._getData(dak);
        delete nrdp._syncData.storage.data[dak];
        nrdp._setProperty("storage", "data", JSON.stringify(nrdp._syncData.storage.data));
    },

    _clearAll: function() {
        nrdp._syncData.storage = {
            data: {}
        };
        nrdp._setProperty("storage", "data", JSON.stringify(nrdp._syncData.storage.data));
    },

    _getData: function(dak) {
        if (dak != this.NO_DEVICE_ACCOUNT)
            nrdp.registration._validateDak(dak);
        if (!nrdp._syncData.storage) {
            nrdp._syncData.storage = {};
        }
        if (!nrdp._syncData.storage.data) {
            nrdp._syncData.storage.data = {};
        }
        if (!nrdp._syncData.storage.data[dak]) {
            nrdp._syncData.storage.data[dak] = {};
        }
        return nrdp._syncData.storage.data[dak];
    },

    _updateProperty: function(property, value) {
        if (property == "data") {
            try {
                nrdp._syncData.storage.data = JSON.parse(value);
            } catch (x) {
                nrdp._syncData.storage.data = {};
            }
            // XXX: should probably clean up storage for device account keys that no longer exist
            // probably won't be able to do that until we get object sync complete
        }
    }
};

nrdp.log = {
    get traceAreas() { return nrdp._syncData.log.traceAreas; },
    set traceAreas(e) { nrdp._setProperty("log", "traceAreas", e); },

    _log: function(level, msg) {
        if (window.nrdp_platform != null && window.nrdp_platform.log != null) {
            nrdp_platform.log(level, msg);
        } else {
            nrdp._invoke("log", "log", {logLevel: level, msg: msg});
        }
    },
    debug: function(str) { nrdp.log._log(20, str); },
    info: function(str) { nrdp.log._log(30, str); },
    error: function(str) { nrdp.log._log(40, str); },
    fatal: function(str) { nrdp.log._log(50, str); },
    trace: function(str) {
        if (nrdp._isReady && !nrdp._isError) {
            nrdp.log._log(10, str);
        } else {
            console.log(str);
        }
    },

    _handleEvent: function(event) {
        if (event.name == "message") {
            console.log("NETFLIX:" + event.data.traceArea + ": " + event.data.msg);
        } else {
            return false;
        }

        return true;
    },

    _updateProperty: function(property, value) {
        if (property == "traceAreas") {
            nrdp._syncData.log.traceAreas = value;
        }
    }
};

nrdp.media = {
    addEventListener: function(evt, listener) { nrdp._addEventListener("media", evt, listener); },
    removeEventListener: function(evt, listener) { nrdp._removeEventListener("media", evt, listener); },

    get audioTrackList() { return nrdp._syncData.media.audioTrackList; },
    get subtitleTrackList() { return nrdp._syncData.media.subtitleTrackList; },
    get currentAudioTrack() { return nrdp._syncData.media.currentAudioTrack; },
    get currentSubtitleTrack() { return nrdp._syncData.media.currentSubtitleTrack; },
    get duration() { return nrdp._syncData.media.duration; },
    get overrideNativeSubtitles() { return nrdp._syncData.media.overrideNativeSubtitles; },
    get state() { return nrdp._syncData.media.state; },
    get xid() { return nrdp._syncData.media.xid; },

    set overrideNativeSubtitles(override) {
        if (nrdp.media.overrideNativeSubtitles != override)
            nrdp._setProperty("media", "overrideNativeSubtitles", override);
    },
    set currentAudioTrack(currentAudioTrack) {
        nrdp.media._setAudioTrack = currentAudioTrack;
        nrdp._setProperty("media", "currentAudioTrack", currentAudioTrack);
    },
    set currentSubtitleTrack(currentSubtitleTrack) {
        nrdp._setProperty("media", "currentSubtitleTrack", currentSubtitleTrack);
    },

    // stream types
    get UNKNOWN_STREAM() { return -1; },
    get AUDIO_STREAM() { return 0; },
    get VIDEO_STREAM() { return 1; },
    get TIMEDTEXT_STREAM() { return 2; },

    // IMediaControl state
    get OPENING() { return 0; },
    get PLAYING() { return 1; },
    get PAUSED() { return 2; },
    get STOPPED() { return 3; },
    get CLOSED() { return 4; },

    // playback types
    get STANDARD_PLAYBACK() { return 1; },
    get BROWSE_PLAYBACK() { return 2; },

    bringToFront: function() { nrdp._invoke("media", "bringToFront"); },
    sendToBack: function() { nrdp._invoke("media", "sendToBack"); },

    open: function(movieId, trackerId, params, playbackType) {
        var args = {movieId: movieId, trackerId: trackerId};
        if (playbackType != undefined && playbackType != null)
            args.originator = playbackType;
        args.params = nrdp._urlEncode(params);
        nrdp._invoke("media", "open", args);
    },

    play: function(ms) {
        var args = {};
        if (ms != undefined && ms != null)
            args.ms = ms;
        if (nrdp.media._setAudioTrack !== undefined) {
            args.track = nrdp.media._setAudioTrack;
            nrdp.media._setAudioTrack = undefined;
        }
        nrdp._invoke("media", "play", args);
    },

    pause: function() { nrdp._invoke("media", "pause"); },
    unpause: function() { nrdp._invoke("media", "unpause"); },
    stop: function() { nrdp._invoke("media", "stop"); },
    close: function() { nrdp._invoke("media", "close"); },

    setVideoWindow: function(x, y, width, height, transitionDuration) {
        var args = {
            x: x,
            y: y,
            width: width,
            height: height,
            transitionDuration: transitionDuration
        };
        nrdp._invoke("media", "setVideoWindow", args);
    },

    getCurrentPts: function(cb) {
        nrdp._invoke("media", "getCurrentPts", null,
                     function(resp) {
                         var obj;
                         try {
                             obj = JSON.parse(resp).returnValue;
                         } catch (x) {
                             nrdp.log.error("unable to parse response to getCurrentPts: " + resp);
                             return;
                         }
                         if (cb) cb(obj);
                     });
    },

    transitionPlaybackType: function() { nrdp._invoke("media", "transitionPlaybackType"); },

    _handleEvent: function(event) {
        if (event.name == "Nccp") {
            if (event.data.origin != "complete")
                event.data.type = "nccperror";
            else
                return true;
        } else if (event.name != "IMediaControl") {
            return false;
        }

        nrdp._callEventListeners("media", event.data);
        return true;
    }
};




////////////////// VIDEO BRIDGE!!!!!!! USE AT YOUR OWN RISK ////////////////////






//------------------------------------------------------------------------------
// Public API
//------------------------------------------------------------------------------

nrdp.video = {
    addEventListener: function(evt, listener, useCapture) { nrdp._addEventListener("video", evt, listener); },
    removeEventListener: function(evt, listener, useCapture) { nrdp._removeEventListener("video", evt, listener); },
    setGlobalEventListener: function(listener) { nrdp.video._priv.globalEventListener = listener; },

    // This is a public API for compatibility with a real HTML5 <video> element
    // but it is not expected to be very useful in our context.
    dispatchEvent: function(event) { nrdp.video._priv.dispatchEvent(event); },

    canPlayType: function() { return "probably"; },

    load: function() { nrdp.video._priv.load(); },

    play: function() { nrdp.video._priv.play(); },

    pause: function() { nrdp.video._priv.pause(); },

    setVideoWindow: function(top, left, width, height, transitionDuration) {
        nrdp.media.setVideoWindow(top, left, width, height, transitionDuration);
    },
    bringToFront: function() { nrdp.media.bringToFront(); },
    sendToBack: function() { nrdp.media.sendToBack(); },

    // HTML5 video attributes

    get left() { return nrdp.video._priv.left; },
    set left(s) { nrdp.media.setVideoWindow(s, nrdp.video._priv.top, nrdp.video._priv.width, nrdp.video._priv.height, 0); },

    get top() { return nrdp.video._priv.top; },
    set top(s) { nrdp.media.setVideoWindow(nrdp.video._priv.left, s, nrdp.video._priv.width, nrdp.video._priv.height, 0); },

    get width() { return nrdp.video._priv.width; },
    set width(s) { nrdp.media.setVideoWindow(nrdp.video._priv.left, nrdp.video._priv.top, s, nrdp.video._priv.height, 0); },

    get height() { return nrdp.video._priv.height; },
    set height(s) { nrdp.media.setVideoWindow(nrdp.video._priv.left, nrdp.video._priv.top, nrdp.video._priv.width, s, 0); },

    get videoWidth() { return nrdp.video._priv.getVideoWidth(); },

    get videoHeight() { return nrdp.video._priv.getVideoHeight(); },

    get aspectRatioX() { return nrdp.video._priv.getAspectRatioX(); },

    get aspectRatioY() { return nrdp.video._priv.getAspectRatioY(); },

    get src() { return nrdp.video._priv.src; },
    set src(s) { nrdp.video._priv.setSrc(s); },

    get currentSrc() { return nrdp.video._priv.currentSrc; },

    get audioTrackList() { return nrdp.media.audioTrackList; },

    get audioTrack() {
        var cur = nrdp.media.currentAudioTrack;
        if (cur !== undefined && cur != -1)
            return "netflix://audioTrack/index=" + cur;
        return "";
    },
    set audioTrack(s) { nrdp.video._priv.setAudioTrack(s); },

    get timedTextTrackList() { return nrdp.media.subtitleTrackList; },

    get timedText() {
        var cur = nrdp.media.currentSubtitleTrack;
        if (cur !== undefined && cur != -1)
            return "netflix://timedText/index=" + cur;
        return "";
    },
    set timedText(s) { nrdp.video._priv.setTimedText(s); },

    // HTML5 status attributes

    get networkState() { return nrdp.video._priv.getNetworkState(); },
    get readyState() { return nrdp.video._priv.getReadyState(); },
    get instantBitRate() { return nrdp.video._priv.instantBitRate; },
    get targetBitRate() { return nrdp.video._priv.getTargetBitRate(); },
    get isHighDefinition() { return nrdp.video._priv.getIsHD(); },
    get forecastedStalls() { return nrdp.video._priv.forecastedStalls; },

    // HTML5 non-applicable attributes

    get poster() { return undefined; },
    set poster(s) {},

    get autobuffer() { return false; },
    set autobuffer(s) {},

    get buffered() { return 0; },

    get startTime() { return 0; },

    get defaultPlaybackRate() { return 1; },
    set defaultPlaybackRate(s) {},

    get playbackRate() { return 1; },
    set playbackRate(s) {},

    get played() { return 0; },

    get seekable() { return 0; },

    get loop() { return false; },
    set loop(s) {},

    get controls() { return false; },
    set controls(s) {},

    get volume() { return 0; },
    set volume(s) {},

    get muted() { return false; },
    set muted(s) {},

    // Media Control

    get duration() { return (nrdp.media.duration / 1000); },

    get currentTime() { return nrdp.video._priv.getCurrentTime() / 1000; },
    set currentTime(sec) { nrdp.video._priv.setCurrentTime(sec * 1000); },

    get seeking() { return nrdp.video._priv.state == nrdp.video._priv.SEEKING; },

    get paused() { return (nrdp.video._priv.state == nrdp.video._priv.PAUSED); },

    get ended() { return nrdp.video._priv.state == nrdp.video._priv.ENDED; },

    get autoplay() { return nrdp.video._priv.autoplay; },
    set autoplay(s) { nrdp.video._priv.setAutoPlay(s); },

    get overrideNativeSubtitles() { return nrdp.media.overrideNativeSubtitles; },
    set overrideNativeSubtitles(s) { nrdp.media.overrideNativeSubtitles = s; },

    get playbackoriginator() { return nrdp.video._priv.originator; },
    transitionPlaybackType: function() { nrdp.video._priv.transitionPlaybackType(); },

    get xid() { return nrdp.media.xid; },

    // Creation & Initialization

    get htmlVideoElement() { return 0; },

    // HTML5 Media Error Codes

    get OK() { return 0; },
    get MEDIA_ERR_ABORTED() { return 1; },
    get MEDIA_ERR_NETWORK() { return 2; },
    get MEDIA_ERR_DECODE() { return 3; },
    get MEDIA_ERR_SRC_NOT_SUPPORTED() { return 4; },

    // HTML5 Network State

    get NETWORK_EMPTY() { return 0; },
    get NETWORK_IDLE() { return 1; },
    get NETWORK_LOADING() { return 2; },
    get NETWORK_NO_SOURCE() { return 3; },

    // HTML5 Ready State

    get HAVE_NOTHING() { return 0; },
    get HAVE_METADATA() { return 1; },
    get HAVE_CURRENT_DATA() { return 2; },
    get HAVE_FUTURE_DATA() { return 3; },
    get HAVE_ENOUGH_DATA() { return 4; },

    get USER() { return "USER"; },
    get BROWSE() { return "BROWSE"; },

    // Event Handlers

    get onloadstart() { return nrdp.video._priv.eventHandlers["loadstart"]; },
    set onloadstart(h) { nrdp.video._priv.eventHandlers["loadstart"] = h; },

    get onprogress() { return nrdp.video._priv.eventHandlers["progress"]; },
    set onprogress(h) { nrdp.video._priv.eventHandlers["progress"] = h; },

    get onsuspend() { return nrdp.video._priv.eventHandlers["suspend"]; },
    set onsuspend(h) { nrdp.video._priv.eventHandlers["suspend"] = h; },

    get onload() { return nrdp.video._priv.eventHandlers["load"]; },
    set onload(h) { nrdp.video._priv.eventHandlers["load"] = h; },

    get onabort() { return nrdp.video._priv.eventHandlers["abort"]; },
    set onabort(h) { nrdp.video._priv.eventHandlers["abort"] = h; },

    get onerror() { return nrdp.video._priv.eventHandlers["error"]; },
    set onerror(h) { nrdp.video._priv.eventHandlers["error"] = h; },

    get onloadend() { return nrdp.video._priv.eventHandlers["loadend"]; },
    set onloadend(h) { nrdp.video._priv.eventHandlers["loadend"] = h; },

    get onemptied() { return nrdp.video._priv.eventHandlers["emptied"]; },
    set onemptied(h) { nrdp.video._priv.eventHandlers["emptied"] = h; },

    get onstalled() { return nrdp.video._priv.eventHandlers["stalled"]; },
    set onstalled(h) { nrdp.video._priv.eventHandlers["stalled"] = h; },

    get onplay() { return nrdp.video._priv.eventHandlers["play"]; },
    set onplay(h) { nrdp.video._priv.eventHandlers["play"] = h; },

    get onpause() { return nrdp.video._priv.eventHandlers["pause"]; },
    set onpause(h) { nrdp.video._priv.eventHandlers["pause"] = h; },

    get onloadedmetadata() { return nrdp.video._priv.eventHandlers["loadedmetadata"]; },
    set onloadedmetadata(h) { nrdp.video._priv.eventHandlers["loadedmetadata"] = h; },

    get onloadeddata() { return nrdp.video._priv.eventHandlers["loadeddata"]; },
    set onloadeddata(h) { nrdp.video._priv.eventHandlers["loadeddata"] = h; },

    get onwaiting() { return nrdp.video._priv.eventHandlers["waiting"]; },
    set onwaiting(h) { nrdp.video._priv.eventHandlers["waiting"] = h; },

    get onplaying() { return nrdp.video._priv.eventHandlers["playing"]; },
    set onplaying(h) { nrdp.video._priv.eventHandlers["playing"] = h; },

    get oncanplay() { return nrdp.video._priv.eventHandlers["canplay"]; },
    set oncanplay(h) { nrdp.video._priv.eventHandlers["canplay"] = h; },

    get oncanplaythrough() { return nrdp.video._priv.eventHandlers["canplaythrough"]; },
    set oncanplaythrough(h) { nrdp.video._priv.eventHandlers["canplaythrough"] = h; },

    get onseeking() { return nrdp.video._priv.eventHandlers["seeking"]; },
    set onseeking(h) { nrdp.video._priv.eventHandlers["seeking"] = h; },

    get onseeked() { return nrdp.video._priv.eventHandlers["seeked"]; },
    set onseeked(h) { nrdp.video._priv.eventHandlers["seeked"] = h; },

    get ontimeupdate() { return nrdp.video._priv.eventHandlers["timeupdate"]; },
    set ontimeupdate(h) { nrdp.video._priv.eventHandlers["timeupdate"] = h; },

    get onended() { return nrdp.video._priv.eventHandlers["ended"]; },
    set onended(h) { nrdp.video._priv.eventHandlers["ended"] = h; },

    get onratechange() { return nrdp.video._priv.eventHandlers["ratechange"]; },
    set onratechange(h) { nrdp.video._priv.eventHandlers["ratechange"] = h; },

    get ondurationchange() { return nrdp.video._priv.eventHandlers["durationchange"]; },
    set ondurationchange(h) { nrdp.video._priv.eventHandlers["durationchange"] = h; },

    get onvolumechange() { return nrdp.video._priv.eventHandlers["volumechange"]; },
    set onvolumechange(h) { nrdp.video._priv.eventHandlers["volumechange"] = h; },

    get onstreamchange() { return nrdp.video._priv.eventHandlers["streamchange"]; },
    set onstreamchange(h) { nrdp.video._priv.eventHandlers["streamchange"] = h; },

    get onaudiotrackchange() { return nrdp.video._priv.eventHandlers["audiotrackchange"]; },
    set onaudiotrackchange(h) { nrdp.video._priv.eventHandlers["audiotrackchange"] = h; },

    get onsubtitletrackchange() { return nrdp.video._priv.eventHandlers["subtitletrackchange"]; },
    set onsubtitletrackchange(h) { nrdp.video._priv.eventHandlers["subtitletrackchange"] = h; },

    get onshowsubtitle() { return nrdp.video._priv.eventHandlers["showsubtitle"]; },
    set onshowsubtitle(h) { nrdp.video._priv.eventHandlers["showsubtitle"] = h; },

    get onremovesubtitle() { return nrdp.video._priv.eventHandlers["removesubtitle"]; },
    set onremovesubtitle(h) { nrdp.video._priv.eventHandlers["removesubtitle"] = h; },

    get onvideowindowchanged() { return nrdp.video._priv.eventHandlers["videowindowchanged"]; },
    set onvideowindowchanged(h) { nrdp.video._priv.eventHandlers["videowindowchanged"] = h; }

};

//------------------------------------------------------------------------------
// Implementation
//------------------------------------------------------------------------------

nrdp.video._priv = {
    /* media control stored state */
    // streamInfo is only valid in LOADED through ENDED
    streamInfo: null,
    isUnderflow: false,
    instantBitRate: 0,
    forecastedStalls: 0,

    time0: 0,
    time1: 0,
    timerID: undefined,

    top: 0,
    left: 0,
    width: screen.width,
    height: screen.height,

    /* what's currently going on */
    CLOSED: "closed",
    LOADING: "loading", // src set but not done loading
    LOADFAIL: "loadfail", // got error from MC during load, still need to close
    LOADED: "loaded", // successfully loaded, no play yet
    SEEKING: "seeking", // moving currentTime (including initial play())
    BUFFERING: "buffering", // buffering after a seek
    REBUFFERING: "rebuffering", // unexpected rebuffering
    PLAYING: "playing", // do you need to ask
    PAUSED: "paused", // seriously
    ENDED: "ended", // end of stream reached
    CLOSING: "closing", // close called and waiting for finish
    state: "closed",

    // CLOSED
    // LOADFAIL
    //     load(valid movie id) -> LOADING
    //     load(invalid movie id) -> CLOSED
    //
    // LOADING
    //     nccperror() -> LOADFAIL
    //     error() -> LOADFAIL
    //     statechanged(CLOSED) -> LOADFAIL
    //     opencomplete() -> LOADED
    //
    // LOADED
    //     play() -> SEEKING
    //
    // SEEKING
    //     buffering() -> BUFFERING
    //
    // BUFFERING
    // REBUFFERING
    //     updatepts() -> PLAYING
    //
    // PLAYING
    //     setCurrentTime() -> SEEKING
    //   expected this state but could come in any state:
    //     endofstream() -> ENDED
    //     pause() -> statechanged(PAUSED) -> PAUSED
    //     buffering() -> REBUFFERING // unexpected
    //
    // PAUSED
    //     setCurrentTime() -> play() -> SEEKING
    //     play() -> PLAYING
    //
    // ENDED
    //     play() -> SEEKING
    //
    // CLOSING
    //     statechanged(CLOSED) -> CLOSED
    //
    // ALL
    //     close() -> CLOSING

    currentSrc: "",
    // not really necessary to keep track of these independent of src,
    // but that way I don't need to parse src repeatedly
    movieId: 0,
    trackerId: -1,
    originator: nrdp.video.USER,
    srcParams: undefined,

    autoplay: false,

    /* what the UI wants to have happen */
    src: "",
    playPending: false,
    pausePending: false,
    pts: undefined,

    //------------------------------------------------------------------------------
    // Getters/setters
    //------------------------------------------------------------------------------

    setState: function(newState) {
        nrdp.log.trace("changing video state from " + this.state + " to " + newState);
        this.state = newState;
    },

    getNetworkState: function() {
        if (this.state == this.LOADING || this.state == this.SEEKING || this.state == this.BUFFERING || this.state == this.REBUFFERING || this.state == this.PLAYING) {
            return nrdp.video.NETWORK_LOADING;
        } else if (this.state == this.LOADFAIL) {
            return nrdp.video.NETWORK_NO_SOURCE;
        } else if (this.state == this.CLOSED) {
            return nrdp.video.NETWORK_EMPTY;
        } else {
            return nrdp.video.NETWORK_IDLE;
        }
    },

    getReadyState: function() {
        if (this.state == this.LOADED) {
            return nrdp.video.HAVE_METADATA;
        } else if (this.state == this.SEEKING || this.state == this.BUFFERING || this.state == this.REBUFFERING) {
            return nrdp.video.HAVE_CURRENT_DATA;
        } else if (this.state == this.PLAYING || this.state == this.PAUSED) {
            return nrdp.video.HAVE_ENOUGH_DATA;
        } else {
            return nrdp.video.HAVE_NOTHING;
        }
    },

    parseUrl: function(url, type, insensitive) {
        var rv = {};

        if (!url) {
            return rv;
        }

        var prefix = "netflix://" + type + "/";
        var reg = new RegExp("^" + prefix, "i");

        if (!reg.test(url)) {
            nrdp.log.trace("parseUrl of " + url + " does not match type " + type);
            return rv;
        }

        var bits = url.substr(prefix.length).split('&');

        for (var i = 0; i < bits.length; i++) {
            var x = bits[i].split('=');
            if (insensitive) {
                var lower = x[0].toLowerCase();
                if (insensitive[lower]) {
                    rv[x[0].toLowerCase()] = x[1];
                } else {
                    rv[x[0]] = x[1];
                }
            } else {
                rv[x[0].toLowerCase()] = x[1];
            }
        }

        return rv;
    },

    setSrc: function(s) {
        nrdp.log.trace("set src: '" + s + "'");

        this.src = s;

        var ps = this.parseUrl(s, "video", {movieid:1, trackerid:1, originator:1});

        if (this.state == this.CLOSED) {
            this.currentSrc = s;
            this.movieId = ps.movieid ? ps.movieid : 0; // never leave it null/undefined
            delete ps.movieid;
            this.trackerId = ps.trackerid ? ps.trackerid : 0;
            delete ps.trackerid;
            if (ps.originator == nrdp.video.BROWSE) {
                this.originator = ps.originator;
            } else {
                this.originator = nrdp.video.USER;
            }
            delete ps.originator;
            this.srcParams = ps;

            if (s) {
                // why do we do an implicit load here? this isn't part of html5
                this.load();
            }
        } else {
            this.close();
        }
    },

    transitionPlaybackType: function() {
        if (this.originator == nrdp.video.BROWSE) {
            this.originator = nrdp.video.USER;
            nrdp.media.transitionPlaybackType();
        }
    },

    setAutoPlay: function(p) {
        this.autoplay = p;
    },

    setAudioTrack: function(url) {
        if (!url) {
            nrdp.log.trace("empty audio track");
            return;
        }

        var params = this.parseUrl(url, "audioTrack");

        if (params["index"] != undefined) {
            nrdp.log.trace("setting audio track to index " + params["index"]);
            nrdp.media.currentAudioTrack = params["index"];
            return;
        }

        if (!params["movieid"]) {
            nrdp.log.trace("missing movie ID");
            return;
        }

        if (params["movieid"] != this.movieId) {
            nrdp.log.trace("wrong movieID set");
            return;
        }

        var tracks = nrdp.media.audioTrackList;
        for (var i = 0; i < tracks.length; i++) {
            if (params["narrative"] != undefined &&
                params["narrative"] != tracks[i].trackType)
                continue;
            if (params["channels"] != undefined &&
                params["channels"] != tracks[i].channels)
                continue;
            if (params["codec"] != undefined &&
                params["codec"] != tracks[i].codecName)
                continue;
            if (params["lang"] != undefined &&
                params["lang"] != tracks[i].language)
                continue;
            if (params["desc"] != undefined &&
                params["desc"] != tracks[i].languageDescription)
                continue;
            nrdp.log.trace("set audio track matched index " + i);
            nrdp.media.currentAudioTrack = i;
            return;
        }

        nrdp.log.trace("audioTrack - no match found");
    },

    setTimedText: function(url) {
        if (!url) {
            nrdp.log.trace("empty subtitle track");
            nrdp.media.currentSubtitleTrack = -1;
            return;
        }

        var params = this.parseUrl(url, "timedText");

        if (params["index"] != undefined) {
            nrdp.log.trace("setting subtitle track to index " + params["index"]);
            nrdp.media.currentSubtitleTrack = params["index"];
            return;
        }

        if (!params["movieid"]) {
            nrdp.log.trace("missing movie ID");
            return;
        }

        if (params["movieid"] != this.movieId) {
            nrdp.log.trace("wrong movieID set");
            return;
        }

        var tracks = nrdp.media.subtitleTrackList;
        for (var i = 0; i < tracks.length; i++) {
            if (params["narrative"] != undefined &&
                params["narrative"] != tracks[i].trackType)
                continue;
            if (params["lang"] != undefined &&
                params["lang"] != tracks[i].language)
                continue;
            if (params["desc"] != undefined &&
                params["desc"] != tracks[i].languageDescription)
                continue;
            nrdp.log.trace("set subtitle track matched index " + i);
            nrdp.media.currentSubtitleTrack = i;
            return;
        }

        nrdp.log.trace("timedtext - no match found");
    },

    getCurrentTime: function() {
        return (this.state == this.PLAYING) ?
            (new Date()).getTime() - this.time0 :
            // if we're rebuffering or paused, time1 is the time we
            // were at before the change; otherwise it's 0
            this.time1;
    },

    setCurrentTime: function(ms) {
        nrdp.log.trace("set currentTime in state " + this.state);
        this.pts = ms;
        if (this.state == this.PLAYING) {
            this.time1 = this.getCurrentTime();
            this.setState(this.SEEKING);
            nrdp.media.stop();
        }
    },

    startTimeTimer: function() {
        var that = this;
        var cb = function() {
            if (that.state != that.PLAYING || nrdp._isError) {
                clearInterval(that.timerID);
                that.timerID = undefined;
                nrdp._setProperty("media", "throttled", false);
                return;
            }
            that.sendMediaEvent("timeupdate");
        };
        if (!this.timerID) {
            nrdp.log.trace("starting timeupdate timer");
            this.timerID = setInterval(cb, 1000);
        }
    },

    getVideoWidth: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.frameWidth;
        }
        return 0;
    },
    getVideoHeight: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.frameHeight;
        }
        return 0;
    },
    getAspectRatioX: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.displayAspectRatioX;
        }
        return 0;
    },
    getAspectRatioY: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.displayAspectRatioY;
        }
        return 0;
    },
    getTargetBitRate: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.bitsPerSecond;
        }
        return 0;
    },
    getIsHD: function() {
        if (this.streamInfo != null) {
            return this.streamInfo.isHighDefinition;
        }
        return false;
    },

    //------------------------------------------------------------------------------
    // Sending events to listeners of nrdp.video
    //------------------------------------------------------------------------------

    globalEventListener: null,
    eventHandlers: [],

    _preventDefault: false,
    dispatchEvent: function(event) {
        this._preventDefault = false;
        event.preventDefault = function() {
            this._preventDefault = true;
        };

        this.callEventHandlersAndListeners(event);

        return this._preventDefault;
    },

    hasEventer: function(type) {
        return (this.globalEventListener ||
                this.eventHandlers[type] ||
                nrdp._hasEventListener("video", type));
    },

    callEventHandlersAndListeners: function(event) {
        nrdp.log.trace("sending event " + event.type + " in state " + this.state);
        if (this.globalEventListener) {
            nrdp.log.trace("call global listener for " + event.type);
            this.globalEventListener(event);
        }

        if (this.eventHandlers[event.type]) {
            nrdp.log.trace("call handler for " + event.type);
	    this.eventHandlers[event.type](event);
        }

        nrdp._callEventListeners("video", event);
    },

    sendEvent: function(event) {
        event.bubbles = false;
        event.cancelable = false;

        if (event.category != "error") {
            nrdp.video.error = null;
        }

        this.callEventHandlersAndListeners(event);
    },

    sendMediaEvent: function(type) {
        if (!this.hasEventer(type)) return;
        var event = {
            category: "media",
            type: type
        };
        this.sendEvent(event);
    },
    sendProgressEvent: function(type, loaded, total) {
        if (!this.hasEventer(type)) return;
        var event = {
            category: "progress",
            type: type,

            lengthComputable: true,
            loaded: loaded,
            total: total,
            loadedItems: 0,
            totalItems: 0
        };
        this.sendEvent(event);
    },
    sendTrackChangeEvent: function(type, track) {
        if (!this.hasEventer(type)) return;
        var event = {
            category: "trackChange",
            type: type,

            detail: track
        };
        event.__defineGetter__("view", function() { return document.window; } );
        this.sendEvent(event);
    },
    sendSubtitleEvent: function(type, subtitle) {
        if (!this.hasEventer(type)) return;
        var event = {
            category: "subtitle",
            type: type,

            data: subtitle
        };
        this.sendEvent(event);
    },
    sendErrorEvent: function(nfErr, text, code) {
        if (this.state == this.CLOSING && nfErr == nrdp.video.NFErr_MC_NCCP_NonRecoverableError) {
            nrdp.log.trace("not sending actionid 1 in closing state");
            return;
        }
        if (!text) text = "";
        if (code == undefined) {
            switch (nfErr) {
            // Standard errors
            default:
            case nrdp.video.NFErr_OK:
            case nrdp.video.NFErr_InvalidStateTransition:
            case nrdp.video.NFErr_MC_SubtitleFailure:
                code = nrdp.video.OK;
                break;
            case nrdp.video.NFErr_MalformedData:
            case nrdp.video.NFErr_MC_AuthFailure:
            case nrdp.video.NFErr_MC_AcquireLicenseFailure:
            case nrdp.video.NFErr_MC_RefuseToPlayNonDrmed:
            case nrdp.video.NFErr_MC_ContentNotAvailable:
                code = nrdp.video.MEDIA_ERR_SRC_NOT_SUPPORTED;
                break;
            case nrdp.video.NFErr_MC_DevicePlaybackError:
            case nrdp.video.NFErr_MC_DeviceDecryptionError:
                code = nrdp.video.MEDIA_ERR_DECODE;
                break;
            case nrdp.video.NFErr_MC_InitStreamFailure:
            case nrdp.video.NFErr_MC_StreamSetIncomplete:
            case nrdp.video.NFErr_MC_OpenDeviceFailure:
            case nrdp.video.NFErr_MC_ConnectionFailure:
            case nrdp.video.NFErr_MC_HttpServerError:
            case nrdp.video.NFErr_MC_HttpClientError:
                code = nrdp.video.MEDIA_ERR_NETWORK;
                break;

            // ActionID errors
            case nrdp.video.NFErr_MC_NCCP_RegistrationRequired:
            case nrdp.video.NFErr_MC_NCCP_CTicketRenewalRequired:
            case nrdp.video.NFErr_MC_NCCP_MTicketRenewalRequired:
            case nrdp.video.NFErr_MC_NCCP_ImpossibleImpossibility:
            case nrdp.video.NFErr_MC_NCCP_GetNewCredentials:
            case nrdp.video.NFErr_MC_NCCP_UnsupportedVersion:
            case nrdp.video.NFErr_MC_NCCP_SecondaryCredentialsRenewalRequired:
            case nrdp.video.NFErr_MC_NCCP_StaleCredentials:
                code = nrdp.video.MEDIA_ERR_SRC_NOT_SUPPORTED;
                break;
            case nrdp.video.NFErr_MC_NCCP_NonRecoverableError:
            case nrdp.video.NFErr_MC_NCCP_CustomError:
            case nrdp.video.NFErr_MC_NCCP_AbortPlayback:
            case nrdp.video.NFErr_MC_NCCP_Exception:
                code = nrdp.video.MEDIA_ERR_DECODE;
                break;
            case nrdp.video.NFErr_MC_NCCP_PotentiallyRecoverableError:
                code = nrdp.video.MEDIA_ERR_NETWORK;
                break;
            }
        }
        nrdp.video.error = {
            code: code,
            nfErr: nfErr,
            text: text
        };
        var event = {
            category: "error",
            type: "error",

            errorCode: code,
            errorNFErr: nfErr,
            errorText: text,

            lengthComputable: false,
            loaded: 0,
            total: 0,
            loadedItems: 0,
            totalItems: 0
        };
        this.sendEvent(event);
    },
    sendActionIdEvent: function(id, message) {
        var nferr;
        switch (id) {
        default:
        case  1: nferr = nrdp.video.NFErr_MC_NCCP_NonRecoverableError;                   break;
        case  2: nferr = nrdp.video.NFErr_MC_NCCP_PotentiallyRecoverableError;           break;
        case  3: nferr = nrdp.video.NFErr_MC_NCCP_CustomError;                           break;
        case  4: nferr = nrdp.video.NFErr_MC_NCCP_RegistrationRequired;                  break;
        case  5: nferr = nrdp.video.NFErr_MC_NCCP_CTicketRenewalRequired;                break;
        case  6: nferr = nrdp.video.NFErr_MC_NCCP_MTicketRenewalRequired;                break;
        case  7: nferr = nrdp.video.NFErr_MC_NCCP_ImpossibleImpossibility;               break;
        case  8: nferr = nrdp.video.NFErr_MC_NCCP_GetNewCredentials;                     break;
        case  9: nferr = nrdp.video.NFErr_MC_NCCP_UnsupportedVersion;                    break;
        case 10: nferr = nrdp.video.NFErr_MC_NCCP_SecondaryCredentialsRenewalRequired;   break;
        case 11: nferr = nrdp.video.NFErr_MC_NCCP_AbortPlayback;                         break;
        case 12: nferr = nrdp.video.NFErr_MC_NCCP_StaleCredentials;                      break;
        }
        this.sendErrorEvent(nferr, message);
    },

    //------------------------------------------------------------------------------
    // Handling nrdp.media events
    //------------------------------------------------------------------------------

    opencomplete: function(event) {
        this.setState(this.LOADED);
        this.sendMediaEvent("loadedmetadata");
        if (this.playPending || this.autoplay) {
            this.playPending = false;
            this.play();
        }
    },
    streamselected: function(event) {
        if (event.streamInfo.streamType == nrdp.media.VIDEO_STREAM) {
            this.streamInfo = event.streamInfo;
            this.forecastedStalls = event.rebuffer;
        }
    },
    newstream: function(event) {
        if (event.streamInfo.streamType == nrdp.media.VIDEO_STREAM) {
            this.streamInfo = event.streamInfo;
            this.sendMediaEvent("streamchange");
        }
    },
    endofstream: function(event) {
        this.time1 = nrdp.media.duration;
        this.setState(this.ENDED);
        this.isUnderflow = false;
        this.sendProgressEvent("load", 100, 100);
        this.sendProgressEvent("loadend", 100, 100);
        this.sendMediaEvent("ended");
    },
    underflow: function(event) {
        this.isUnderflow = true;
        this.sendProgressEvent("stalled", 0, 0);
    },
    buffering: function(event) {
        if (this.state != this.BUFFERING && this.state != this.REBUFFERING) {
            if (this.state == this.SEEKING) {
                this.setState(this.BUFFERING);
            } else {
                this.setState(this.REBUFFERING);
            }
            this.sendMediaEvent("waiting");
        }
        this.sendProgressEvent("progress", event.percentage, 100);
    },
    bufferingcomplete: function(event) {
        nrdp._setProperty("media", "throttled", false);
        this.isUnderflow = false;
        if (this.state == this.SEEKING || this.state == this.BUFFERING) {
            this.sendMediaEvent("seeked");
        }
        this.sendMediaEvent("loadeddata");
    },
    updatepts: function(event) {
        if (this.state == this.CLOSING) {
            // if we told media control to close it's possible there's
            // still a pending updatepts event
            return;
        }
        // we may get updatepts in seeking if there was no need to rebuffer
        if (this.state == this.SEEKING) {
            this.sendMediaEvent("seeked");
        }
        if (this.state != this.PLAYING) {
            var sendEvents = (this.state != this.PAUSED);
            this.setState(this.PLAYING);
            if (sendEvents) {
                // send this here instead of in bufferingcomplete so
                // that the state is PLAYING when it's sent
                this.sendMediaEvent(this.forecastedStalls == 0 ? "canplaythrough" : "canplay");
            }
            this.sendMediaEvent("playing");
            if (!this.vox) {
                nrdp._setProperty("media", "throttled", true);
                this.startTimeTimer();
            }
        }
        this.time0 = (new Date()).getTime() - event.pts;
        this.sendMediaEvent("timeupdate");
    },
    updatevideobitrate: function(event) {
        this.instantBitRate = event.bitsPerSecond;
    },
    audiotrackchanged: function(event) {
        this.sendTrackChangeEvent("audiotrackchange", event.trackIndex);
    },
    subtitletrackchanged: function(event) {
        this.sendTrackChangeEvent("subtitletrackchange", event.trackIndex);
    },
    showsubtitle: function(event) {
        this.sendSubtitleEvent("showsubtitle", event.subtitleID + ":" + event.text);
    },
    removesubtitle: function(event) {
        this.sendSubtitleEvent("removesubtitle", event.subtitleID + "");
    },
    nccperror: function(event) {
        if (event.origin == "handleActionId" || event.origin == "stopTransaction") {
            this.sendActionIdEvent(event.actionId, event.message);
        } else if (event.origin == "reportNetworkingError") {
            this.sendErrorEvent(nrdp.video.NFErr_MC_ConnectionFailure);
        } else {
            return;
        }
        if (this.state != this.CLOSING) {
            this.playPending = false;
            this.pausePending = false;
        }
        if (this.state == this.LOADING) {
            this.setState(this.LOADFAIL);
        }
        this.sendProgressEvent("loadend", 0, 0);
    },
    error: function(event) {
        if ((event.error >= nrdp.video.NFErr_MC_NCCP_NonRecoverableError && event.error <= nrdp.video.NFErr_MC_NCCP_Exception) ||
            event.error == nrdp.video.NFErr_MC_AuthFailureNCCP || event.error == nrdp.video.NFErr_MC_AcquireLicenseFailureNCCP) {
            return;
        }
        this.sendErrorEvent(event.error);
        if (this.state != this.CLOSING) {
            this.playPending = false;
            this.pausePending = false;
        }
        if (this.state == this.LOADING) {
            this.setState(this.LOADFAIL);
        }
        this.sendProgressEvent("loadend", 0, 0);
    },
    warning: function(event) {
        this.sendErrorEvent(event.error, "", nrdp.video.OK);
    },
    exception: function(event) {
        this.sendErrorEvent(nrdp.video.NFErr_MC_NCCP_Exception, event.exception);
        if (this.state != this.CLOSING) {
            this.playPending = false;
            this.pausePending = false;
        }
        if (this.state == this.LOADING) {
            this.setState(this.LOADFAIL);
        }
        this.sendProgressEvent("loadend", 0, 0);
    },
    statechanged: function(event) {
        nrdp.log.trace("media.state changed to " + event.state + " while video in state " + this.state);
        nrdp._setProperty("media", "throttled", false);
        switch (event.state) {
        case nrdp.media.CLOSED:
            var setsrc = (this.state == this.CLOSING);
            this.setState(this.CLOSED);
            this.movieId = 0;
            this.time1 = 0;
            this.sendProgressEvent("loadend", 0, 0);
            if (setsrc) {
                nrdp.log.trace("now that we're closed, setting currentSrc to previously set src: '" + this.src + "'");
                this.setSrc(this.src);
            }
            break;

        case nrdp.media.PLAYING:
            if (this.pausePending) {
                this.pausePending = false;
                nrdp.media.pause();
            } else {
                this.sendMediaEvent("play");
            }
            break;

        case nrdp.media.PAUSED:
            this.time1 = this.getCurrentTime();
            this.setState(this.PAUSED);
            this.sendMediaEvent("timeupdate");
            this.sendMediaEvent("pause");
            break;

        case nrdp.media.STOPPED:
            if (this.state == this.SEEKING) {
                this.play();
            }
        }
    },
    videowindowchanged: function(event) {
        this.left = event.x;
        this.top = event.y;
        this.width = event.width;
        this.height = event.height;
        this.sendMediaEvent("videowindowchanged");
    },

    //------------------------------------------------------------------------------
    // Commands on the video object
    //------------------------------------------------------------------------------

    load: function() {
        nrdp.log.trace("video.load() starting in state " + this.state);

        if (this.state == this.LOADING) {
            nrdp.log.trace("video.load() already in progress");
            return;
        }

        if (this.state == this.LOADFAIL) {
            this.close();
            return;
        }

        this.setState(this.LOADING);
        this.sendProgressEvent("loadstart", 0, 0);

        if (this.movieId != 0) {
            this.streamInfo = null;
            this.forecastedStalls = 0;
            var pbtype = (this.originator == nrdp.video.BROWSE ? nrdp.media.BROWSE_PLAYBACK : nrdp.media.STANDARD_PLAYBACK);
            nrdp.media.open(this.movieId, this.trackerId, this.srcParams, pbtype);
        } else {
            this.setState(this.CLOSED);
            this.sendMediaEvent("emptied");
            this.sendErrorEvent(nrdp.video.NFErr_MalformedData);
            this.sendProgressEvent("loadend", 0, 0);
        }
    },

    play: function() {
        nrdp.log.trace("video.play() starting in state " + this.state);
        if (this.movieId == 0) {
            nrdp.log.trace("invalid media source for play");
            this.sendMediaEvent("emptied");
            this.sendErrorEvent(nrdp.video.NFErr_MalformedData);
            this.sendProgressEvent("loadend", 0, 0);
            return;
        }

        if (this.state == this.LOADING || this.state == this.CLOSING || this.state == this.CLOSED) {
            this.playPending = true;
            nrdp.log.trace("video.play() still wating");
            return;
        }

        if (this.state == this.PAUSED) {
            if (this.pts == undefined) {
                nrdp.media.unpause();
                return;
            } else {
                this.setState(this.SEEKING);
                nrdp.media.stop();
                return;
            }
        }

        if (this.state == this.ENDED) {
            if (this.pts == undefined) {
                this.pts = 0;
            }
            this.setState(this.SEEKING);
            nrdp.media.stop();
            return;
        }

        this.time1 = this.pts || 0;
        this.isUnderflow = false;

        this.setState(this.SEEKING);
        nrdp.media.play(this.pts);
        this.pts = undefined;
        this.sendMediaEvent("seeking");
    },

    pause: function() {
        nrdp.log.trace("video.pause() starting in state " + this.state);
        if (this.state == this.PLAYING) {
            nrdp.media.pause();
        } else if (this.state == this.PAUSED) {
            nrdp.log.trace("pause while already paused");
        } else if (this.state == this.CLOSED || this.state == this.LOADFAIL) {
            nrdp.log.trace("pause while closed");
        }
    },

    close: function() {
        nrdp.log.trace("video.close() starting in state " + this.state);

        if (this.state == this.CLOSING || this.state == this.CLOSED) {
            nrdp.log.trace("video.close() already in progress");
            return;
        }

        this.setState(this.CLOSING);
        this.streamInfo = null;
        this.instantBitRate = 0;
        this.time1 = 0;
        this.forecastedStalls = 0;

        nrdp.media.close();
        // when it's done closing we'll do another setSrc.
    },

    //------------------------------------------------------------------------------
    // Updating our state from sync dump
    //------------------------------------------------------------------------------

    init: function(event) {

        var nferr;
        for (nferr in nrdp._syncData.media.errorCodes) {
            nrdp.video[nferr] = nrdp._syncData.media.errorCodes[nferr];
        }

        nrdp.media.addEventListener("opencomplete", function(evt) { nrdp.video._priv.opencomplete(evt); });
        nrdp.media.addEventListener("streamselected", function(evt) { nrdp.video._priv.streamselected(evt); });
        nrdp.media.addEventListener("newstream", function(evt) { nrdp.video._priv.newstream(evt); });
        nrdp.media.addEventListener("endofstream", function(evt) { nrdp.video._priv.endofstream(evt); });
        nrdp.media.addEventListener("underflow", function(evt) { nrdp.video._priv.underflow(evt); });
        nrdp.media.addEventListener("buffering", function(evt) { nrdp.video._priv.buffering(evt); });
        nrdp.media.addEventListener("bufferingcomplete", function(evt) { nrdp.video._priv.bufferingcomplete(evt); });
        nrdp.media.addEventListener("updatepts", function(evt) { nrdp.video._priv.updatepts(evt); });
        nrdp.media.addEventListener("updatevideobitrate", function(evt) { nrdp.video._priv.updatevideobitrate(evt); });
        nrdp.media.addEventListener("audiotrackchanged", function(evt) { nrdp.video._priv.audiotrackchanged(evt); });
        nrdp.media.addEventListener("subtitletrackchanged", function(evt) { nrdp.video._priv.subtitletrackchanged(evt); });
        nrdp.media.addEventListener("showsubtitle", function(evt) { nrdp.video._priv.showsubtitle(evt); });
        nrdp.media.addEventListener("removesubtitle", function(evt) { nrdp.video._priv.removesubtitle(evt); });
        nrdp.media.addEventListener("nccperror", function(evt) { nrdp.video._priv.nccperror(evt); });
        nrdp.media.addEventListener("error", function(evt) { nrdp.video._priv.error(evt); });
        nrdp.media.addEventListener("warning", function(evt) { nrdp.video._priv.warning(evt); });
        nrdp.media.addEventListener("exception", function(evt) { nrdp.video._priv.exception(evt); });
        nrdp.media.addEventListener("statechanged", function(evt) { nrdp.video._priv.statechanged(evt); });
        nrdp.media.addEventListener("videowindowchanged", function(evt) { nrdp.video._priv.videowindowchanged(evt); });

        // XXX we need to update here based on state
        if (nrdp.media.state == nrdp.media.OPENING)
            this.setState(this.LOADING);
        else if (nrdp.media.state == nrdp.media.PLAYING)
            this.setState(this.PLAYING);
        else if (nrdp.media.state == nrdp.media.PAUSED)
            this.setState(this.PAUSED);
        else if (nrdp.media.state == nrdp.media.STOPPED)
            this.setState(this.ENDED);
        else if (nrdp.media.state == nrdp.media.CLOSED)
            this.setState(this.CLOSED);
        if (this.state != this.CLOSED) {
            this.currentSrc = "netflix://video/movieid=00000001&trackerid=-1";
            this.movieId = 1;
            this.src = this.currentSrc;
        }
    }
};

nrdp.addEventListener("init", function(evt) { nrdp.video._priv.init(evt); });

})();

(function() {

//////
// 3.0 compat
//////

if (typeof nrdp_platform !== "undefined" && nrdp_platform.compatibility == "3.0.0") {
    var SpecialHack = function(evt) {
        console.log("STARTING 3.0 COMPAT HACK");
        nrdp.removeEventListener("init", SpecialHack);
        for (var i = 0; i < nrdp.registration.deviceAccounts.length; i++) {
            if (nrdp.registration.deviceAccounts[i].tokens && nrdp.registration.deviceAccounts[i].tokens.NetflixId) {
                nrdp.registration.selectDeviceAccount(nrdp.registration.deviceAccounts[0].accountKey);
                return;
            }
        }
    };

    nrdp.__init = nrdp.init;
    nrdp.init = undefined;

    document.addEventListener("DOMContentLoaded",
                              function() {
                                  nrdp.addEventListener("init", SpecialHack);
                                  nrdp.__init();
                              }, false);

    var _getProperty = function(obj, prop) {
        obj = obj ? "nrdp." + obj : "nrdp";
        var params = "object=" + obj + "&property=" + prop;
        var xhr = new XMLHttpRequest();
        xhr.open("POST", nrdp._URL + "ipc", false);
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        try {
            xhr.send(params);
            return JSON.parse(xhr.responseText).value;
        } catch (err) {
            return undefined;
        }
    };

    nrdp.nrdlib = {
        _cmp: undefined,

        get xid() { return nrdp.video.xid; },

        ping: function() { if (nrdp.nrdlib._cmp) setTimeout(nrdp.nrdlib._cmp, 1500); },
        cancelPing: function() {},

        actionIdReceived: {
            connect: function(cb) {}
        },
        networkError: {
            connect: function(cb) {}
        },
        pingComplete: {
            connect: function(cb) { nrdp.nrdlib._cmp = cb; }
        }
    };

    nrdp._qt3CompatObject = {
        connect: function(func) { }
    };

    nrdp.webapi = {
        getConsumerKey: function() { return "mjxyfgjrux7bya2f8p55k7we"; }
    };

    nrdp.device.getSoftwareVersion = function() { return _getProperty("device", "softwareVersion"); };
    nrdp.device.getCertificationVersion = function() { return _getProperty("device", "certificationVersion"); };
    nrdp.device.getESNPrefix = function() { return _getProperty("device", "ESNPrefix"); };
    nrdp.device.getSDKVersion = function() { return _getProperty("device", "SDKVersion"); };
    nrdp.device.getESN = function() { return _getProperty("device", "ESN"); };
    nrdp.device.getLanguage = function() { return _getProperty("device", "language"); };
    nrdp.device.getVideoProfiles = function() { return this.capability.videoProfiles; };
    nrdp.device.getAudioProfiles = function() { return this.capability.audioProfiles; };
    nrdp.device.getSubtitleProfiles = function() { return this.capability.subtitleProfiles; };
    nrdp.device.hasPointer = function() { return this.capability.hasPointer; };
    nrdp.device.hasKeyboard = function() { return this.capability.hasKeyboard; };
    nrdp.device.hasOnScreenKeyboard = function() { return this.capability.hasOnScreenKeyboard; };
    nrdp.device.registrationStatusReceived = nrdp._qt3CompatObject;

    nrdp.device.deactivate = function(cb) {
        nrdp.registration.deactivate(nrdp.registration.currentDeviceAccount, cb);
    };
    nrdp.device.resetApplication = function(cb) {
        nrdp.registration.deactivateAll(cb);
    };

    nrdp.video.__defineSetter__("playbackoriginator", function(v) {});
}

})();
